--  ReactionBar 1.1.5 - 11th April 2006
--  Author:  Bastle <vF> Blackrock(US)
--  Website: http://www.vfclan.com

REACTIONBAR_VERSION = "1.1.5"
REACTIONBAR_NAME = "ReactionBar"
REACTIONBAR_NAME_PRINT = "|cff20ff20"..REACTIONBAR_NAME.." "..REACTIONBAR_VERSION.."|r"

REACTIONBAR_ENABLED = true

REACTIONBAR_TEXT_DUEL_STARTING = string.format(DUEL_COUNTDOWN, 3)

REACTIONBAR_MIN_BAR_ID = 1
REACTIONBAR_MAX_BAR_ID = 6
REACTIONBAR_SHAPESHIFT_BAR = 1
REACTIONBAR_DEFAULT_FRIENDLY_BAR = 2
REACTIONBAR_DEFAULT_ENEMY_BAR = 1
REACTIONBAR_DEFAULT_EXTRA_BAR = 3
REACTIONBAR_DEFAULT_RANGE_BAR = 4
REACTIONBAR_DEFAULT_MONITOR_DUEL = true

local playerName
local realmName
local playerClass
local englishClass
local druidDisable
local extraDisable
local duelTarget
local duelStarted
local prevShapeshiftPageID
local barID
local initialised
local updateInterval = TOOLTIP_UPDATE_TIME

ReactionBarDetails = {
	name = REACTIONBAR_NAME,
	version = REACTIONBAR_VERSION,
	author = "Bastle",
   releaseDate = "January 18, 2006",
	email = "vfbastle@gmail.com",
	website = "http://www.vfclan.com",
	category = MYADDONS_CATEGORY_BARS
}

function ReactionBar_OnLoad()
	SLASH_REACTIONBAR1 = "/reactionbar"
	SLASH_REACTIONBAR2 = "/rb"
	SlashCmdList["REACTIONBAR"] = function( msg )
		ReactionBar_SlashCommandHandler( msg )
	end
		
	ReactionBarHelp = {}
	local help = ""
	for _,line in REACTIONBAR_HELP_PAGE do
		help = help.."\n"..line
	end
	ReactionBarHelp[1] = help
	this:RegisterEvent("ADDON_LOADED")
	this:RegisterEvent("VARIABLES_LOADED")
	this:RegisterEvent("UNIT_MODEL_CHANGED")
	this:RegisterEvent("UNIT_FACTION")
	this:RegisterEvent("UNIT_DYNAMIC_FLAGS")
	this:RegisterEvent("PLAYER_TARGET_CHANGED")
end

function ReactionBar_SlashCommandHandler( msg )
	if (string.lower(msg) == REACTIONBAR_SLASH_ENABLE) then
		ReactionBar_Enable(true, 0)
	elseif (string.lower(msg) == REACTIONBAR_SLASH_DISABLE) then
		ReactionBar_Enable(false, 0)
	elseif (string.lower(msg) == REACTIONBAR_SLASH_STATUS) then
		ReactionBar_Status()
	elseif (string.lower(msg) == REACTIONBAR_SLASH_RESET) then
		ReactionBar_Reset()
	elseif (string.lower(msg) == REACTIONBAR_SLASH_MONITOR_DUEL) then
		ReactionBar_MonitorDuel()
	elseif (string.lower(msg) == REACTIONBAR_SLASH_FRIENDLY) then
		ReactionBar_SetFriendlyBar(CURRENT_ACTIONBAR_PAGE)
	elseif (string.lower(msg) == REACTIONBAR_SLASH_ENEMY) then
		ReactionBar_SetEnemyBar(CURRENT_ACTIONBAR_PAGE)
	elseif (string.lower(msg) == REACTIONBAR_SLASH_EXTRA) then
		ReactionBar_SetExtraBar(CURRENT_ACTIONBAR_PAGE)
	elseif (string.lower(msg) == REACTIONBAR_SLASH_RANGE) then
		ReactionBar_SetRangeBar(CURRENT_ACTIONBAR_PAGE)
	elseif (string.find(msg, REACTIONBAR_SLASH_FRIENDLY)) then
		ReactionBar_SetFriendlyBar(string.sub(msg, string.len(REACTIONBAR_SLASH_FRIENDLY)+2))
	elseif (string.find(msg, REACTIONBAR_SLASH_ENEMY)) then
		ReactionBar_SetEnemyBar(string.sub(msg, string.len(REACTIONBAR_SLASH_ENEMY)+2))
	elseif (string.find(msg, REACTIONBAR_SLASH_NOTARGET)) then
		ReactionBar_SetNoTargetBar(string.sub(msg, string.len(REACTIONBAR_SLASH_NOTARGET)+2))
	elseif (string.find(msg, REACTIONBAR_SLASH_EXTRA)) then
		ReactionBar_SetExtraBar(string.sub(msg, string.len(REACTIONBAR_SLASH_EXTRA)+2))
	elseif (string.lower(msg) == nil or string.lower(msg) == "" or string.lower(msg) == REACTIONBAR_SLASH_HELP) then
		table.foreach(REACTIONBAR_HELP_PAGE, function(k,v) DEFAULT_CHAT_FRAME:AddMessage(v) end)
	else
		DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_INVALID_SLASH)
	end
end

function ReactionBar_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		playerClass, englishClass = UnitClass("player")
		playerName = UnitName("player")
		realmName = GetCVar("realmName")
		ReactionBar_Initialise()
		return
	end

	if (event == "ADDON_LOADED") then
		if (arg1 == "ReactionBar" and myAddOnsFrame_Register) then				
			myAddOnsFrame_Register(ReactionBarDetails, ReactionBarHelp)
		end
		return
	end

	if (not extraDisable and not druidDisable and ReactionBarOptions[realmName][playerName].enabled) then
		if ( event == "UNIT_FACTION" ) then
			if ( arg1 == "target" or arg1 == "player" ) then ReactionBar_CheckFaction() end
			return
		end
		if ( event == "UNIT_DYNAMIC_FLAGS" and arg1 == "target" ) then ReactionBar_CheckFaction()
			return
		end
		if ( event == "PLAYER_TARGET_CHANGED") then	
			ReactionBar_CheckFaction()
			return
		end
	end

	if (ReactionBarOptions[realmName][playerName].monitorDuel and ReactionBarOptions[realmName][playerName].enabled) then
		if ( event == "CHAT_MSG_SYSTEM") then
			if (arg1 == ERR_DUEL_REQUESTED) then
				duelTarget = UnitName("target")
				return
			end
			if (arg1 == REACTIONBAR_TEXT_DUEL_STARTING) then
				duelStarted = true
				ReactionBar_CheckFaction()
				return
			end
		end
		if ( event == "DUEL_REQUESTED") then
			duelTarget = arg1
			return
		end
		if ( event == "DUEL_FINISHED") then
			duelStarted = false
			duelTarget = nil
			return
		end
	end
	
	if (event == "UNIT_MODEL_CHANGED" and englishClass == "DRUID" and arg1 == "player") then
		ReactionBar_CheckShapeShift()
		return
	end	
end

function ReactionBar_Reset()
	ReactionBarOptions[realmName][playerName].enabled = true
	ReactionBarOptions[realmName][playerName].friendlyBar = REACTIONBAR_DEFAULT_FRIENDLY_BAR
	ReactionBarOptions[realmName][playerName].enemyBar = REACTIONBAR_DEFAULT_ENEMY_BAR
	ReactionBarOptions[realmName][playerName].extraBar = REACTIONBAR_DEFAULT_EXTRA_BAR
	ReactionBarOptions[realmName][playerName].rangeBar = REACTIONBAR_DEFAULT_RANGE_BAR
	ReactionBarOptions[realmName][playerName].noTargetBar = REACTIONBAR_DEFAULT_FRIENDLY_BAR
	ReactionBarOptions[realmName][playerName].monitorDuel = REACTIONBAR_DEFAULT_MONITOR_DUEL
		
	ReactionBar_SetOptions()
	DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_RESET)
	ReactionBar_Status()	
end

function ReactionBar_Initialise()
	if (ReactionBarOptions == nil) then
		ReactionBarOptions = {}
	end
	if (ReactionBarOptions[realmName] == nil) then
		ReactionBarOptions[realmName] = {}
	end
	if (ReactionBarOptions[realmName][playerName] == nil) then
		ReactionBarOptions[realmName][playerName] = {}
		ReactionBarOptions[realmName][playerName].enabled = true
		for num, class in REACTIONBAR_RESTRICT do	
			if (class == englishClass ) then
				ReactionBarOptions[realmName][playerName].enabled = false
			end
		end
		ReactionBarOptions[realmName][playerName].friendlyBar = REACTIONBAR_DEFAULT_FRIENDLY_BAR
		ReactionBarOptions[realmName][playerName].enemyBar = REACTIONBAR_DEFAULT_ENEMY_BAR
		ReactionBarOptions[realmName][playerName].extraBar = REACTIONBAR_DEFAULT_EXTRA_BAR
		ReactionBarOptions[realmName][playerName].rangeBar = REACTIONBAR_DEFAULT_RANGE_BAR
		ReactionBarOptions[realmName][playerName].noTargetBar = REACTIONBAR_DEFAULT_FRIENDLY_BAR
		ReactionBarOptions[realmName][playerName].monitorDuel = REACTIONBAR_DEFAULT_MONITOR_DUEL
	end
	
	if (ReactionBarOptions[realmName][playerName].extraBar == nil) then
		local set = 0 local id = REACTIONBAR_MIN_BAR_ID
		while (set == 0 and id <= REACTIONBAR_MAX_BAR_ID) do
			if (id == ReactionBarOptions[realmName][playerName].enemyBar or 
			   id == ReactionBarOptions[realmName][playerName].friendlyBar) then
				id = id+1
			else
				ReactionBarOptions[realmName][playerName].extraBar = id
				set = 1
			end
		end
	end
	
	if (ReactionBarOptions[realmName][playerName].rangeBar == nil) then
		local set = 0 local id = REACTIONBAR_MIN_BAR_ID
		while (set == 0 and id <= REACTIONBAR_MAX_BAR_ID) do
			if (id == ReactionBarOptions[realmName][playerName].enemyBar or 
			   id == ReactionBarOptions[realmName][playerName].friendlyBar or 
			   id == ReactionBarOptions[realmName][playerName].extraBar) then
				id = id+1
			else
				ReactionBarOptions[realmName][playerName].rangeBar = id
				set = 1
			end
		end
	end
	
	if (ReactionBarOptions[realmName][playerName].noTargetBar == nil) then
		ReactionBarOptions[realmName][playerName].noTargetBar = ReactionBarOptions[realmName][playerName].friendlyBar
	end
	
	duelStarted = false
	ReactionBar_SetOptions()
	initialised = true
end

function ReactionBar_SetOptions()
	ReactionBar_SetFriendlyBar(ReactionBarOptions[realmName][playerName].friendlyBar)
	ReactionBar_SetEnemyBar(ReactionBarOptions[realmName][playerName].enemyBar)
	ReactionBar_SetExtraBar(ReactionBarOptions[realmName][playerName].extraBar)
	ReactionBar_SetRangeBar(ReactionBarOptions[realmName][playerName].rangeBar)
	ReactionBar_SetNoTargetBar(ReactionBarOptions[realmName][playerName].noTargetBar)
	ReactionBar_MonitorDuel(ReactionBarOptions[realmName][playerName].monitorDuel)

	ReactionBar_Enable(ReactionBarOptions[realmName][playerName].enabled, 0)
	if (ReactionBarOptions[realmName][playerName].enabled) then
		ReactionBar_CheckShapeShift()
	end
end

function ReactionBar_Status()	
	DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_STATUS)
	if (ReactionBarOptions[realmName][playerName].enabled) then
		DEFAULT_CHAT_FRAME:AddMessage(" - "..REACTIONBAR_TEXT_ENABLED)
	else
		DEFAULT_CHAT_FRAME:AddMessage(" - "..REACTIONBAR_DISABLED)
	end
	if (ReactionBarOptions[realmName][playerName].monitorDuel) then
		DEFAULT_CHAT_FRAME:AddMessage(" - "..REACTIONBAR_MONITOR_DUEL.." "..REACTIONBAR_TEXT_ENABLED)
	else
		DEFAULT_CHAT_FRAME:AddMessage(" - "..REACTIONBAR_MONITOR_DUEL.." "..REACTIONBAR_DISABLED)
	end
	DEFAULT_CHAT_FRAME:AddMessage(" - "..REACTIONBAR_FRIENDLYBAR.." "..ReactionBarOptions[realmName][playerName].friendlyBar)
	DEFAULT_CHAT_FRAME:AddMessage(" - "..REACTIONBAR_ENEMYBAR.." "..ReactionBarOptions[realmName][playerName].enemyBar)
	DEFAULT_CHAT_FRAME:AddMessage(" - "..REACTIONBAR_EXTRABAR.." "..ReactionBarOptions[realmName][playerName].extraBar)
	DEFAULT_CHAT_FRAME:AddMessage(" - "..REACTIONBAR_NOTARGETBAR.." "..ReactionBarOptions[realmName][playerName].noTargetBar)
	if (englishClass == "HUNTER") then
		DEFAULT_CHAT_FRAME:AddMessage(" - "..REACTIONBAR_RANGEBAR.." "..ReactionBarOptions[realmName][playerName].rangeBar)
	end
end

function ReactionBar_CheckShapeShift()
	local active = 0 local i = 1 local icon, name
	while (active == 0 and i <= GetNumShapeshiftForms()) do
		icon, name, active = GetShapeshiftFormInfo(i)
		if (active == nil) then active = 0 end
		i = i+1
	end
	if (active == 1) then ReactionBar_Enable(false, 1)
	else ReactionBar_Enable(true, 1) end
end

function ReactionBar_CheckFaction(hunter)
	if (not extraDisable and not druidDisable and ReactionBarOptions[realmName][playerName].enabled) then
		local temp_page = CURRENT_ACTIONBAR_PAGE
		if (ReactionBar_IsEnemy()) then
			temp_page = ReactionBarOptions[realmName][playerName].enemyBar	
			if (englishClass == "HUNTER" and not CheckInteractDistance("target", 1)) then
				temp_page = ReactionBarOptions[realmName][playerName].rangeBar
			end
			if (hunter ~= 1 and englishClass == "HUNTER") then			
				ReactionBarHunterFrame:Show()
			end
		elseif hunter ~= 1 then
		  if ( UnitExists("target") and not UnitIsDead("target") ) then
				temp_page = ReactionBarOptions[realmName][playerName].friendlyBar
			else
				temp_page = ReactionBarOptions[realmName][playerName].noTargetBar
			end
			if (ReactionBarHunterFrame:IsVisible()) then ReactionBarHunterFrame:Hide() end
		end
		ReactionBar_SetActionPage(temp_page)
	end
end

function ReactionBar_ButtonUp(id)
	if (not extraDisable and not druidDisable and ReactionBarOptions[realmName][playerName].enabled) then
		local button, actionPrefix
		if (GBars_State) then actionPrefix = "GActionButton"
		else actionPrefix = "ActionButton" end
		button = getglobal(actionPrefix..id)
		local buttonID = button:GetID() + barID
		if (button:GetButtonState() == "PUSHED") then
			button:SetButtonState("NORMAL")
			if (MacroFrame_SaveMacro) then MacroFrame_SaveMacro() end
			UseAction(buttonID, 0, 1)
			if (IsCurrentAction(buttonID)) then
				button:SetChecked(1)
			else
				button:SetChecked(0)
			end
		end
	else
		ActionButtonUp(id, 1)
	end
end

function ReactionBar_Enable(enable, temp)
	if (enable) then
		druidDisable = false
		extraDisable = false
		duelTarget = nil
		ReactionBarOptions[realmName][playerName].enabled = true
		REACTIONBAR_ENABLED = true
		if (englishClass == "HUNTER") then
			ReactionBarHunterFrame:Show()
		end
		ReactionBar_CheckFaction()
		if (temp == 0) then
			DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_TEXT_ENABLED)
		end
	else
		if (temp == 1) then
			druidDisable = true
			ReactionBar_SetActionPage(REACTIONBAR_SHAPESHIFT_BAR)
		else
			ReactionBarOptions[realmName][playerName].enabled = false
			REACTIONBAR_ENABLED = false
			ReactionBarHunterFrame:Hide()
			DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_DISABLED)
		end
	end
end

function ReactionBar_ToggleExtraPage(show)
	if (ReactionBarOptions[realmName][playerName].enabled) then
		if (show == 1) then
			extraDisable = true
			prevShapeshiftPageID = CURRENT_ACTIONBAR_PAGE
			ReactionBar_SetActionPage(ReactionBarOptions[realmName][playerName].extraBar)
		else
			extraDisable = false
			if (druidDisable) then
				ReactionBar_SetActionPage(prevShapeshiftPageID)
			else
				ReactionBar_CheckFaction()
			end
		end
	end
end

function ReactionBar_MonitorDuel(enable)
	if (enable == nil) then
		DEFAULT_CHAT_FRAME:AddMessage("enable is nil")
		if (ReactionBarOptions[realmName][playerName].monitorDuel) then
			enable = false
		else
			enable = true
		end
	end
	if (enable) then
		ReactionBarOptions[realmName][playerName].monitorDuel = true
		this:RegisterEvent("CHAT_MSG_SYSTEM")
		this:RegisterEvent("DUEL_REQUESTED")
		this:RegisterEvent("DUEL_FINISHED")
	else
		ReactionBarOptions[realmName][playerName].monitorDuel = false
		this:UnregisterEvent("CHAT_MSG_SYSTEM")
		this:UnregisterEvent("DUEL_REQUESTED")
		this:UnregisterEvent("DUEL_FINISHED")
	end
	ReactionBar_Status()
end

function ReactionBar_SetFriendlyBar(id)
	id = tonumber(id)
	if (id == nil) then
		DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_NIL)
		ReactionBar_Status()
		return
	end
	
	if (id >= REACTIONBAR_MIN_BAR_ID and id <= REACTIONBAR_MAX_BAR_ID) then
		ReactionBarOptions[realmName][playerName].friendlyBar = id
		barID = (ReactionBarOptions[realmName][playerName].friendlyBar - 1) * NUM_ACTIONBAR_BUTTONS
	end
	DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_FRIENDLYBAR.." "..id)
	ReactionBar_CheckFaction()
end

function ReactionBar_SetEnemyBar(id)
	id = tonumber(id)
	if (id == nil) then
		DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_NIL)
		ReactionBar_Status()
		return
	end
	
	if (id >= REACTIONBAR_MIN_BAR_ID and id <= REACTIONBAR_MAX_BAR_ID) then
		ReactionBarOptions[realmName][playerName].enemyBar = id
	end
	DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_ENEMYBAR.." "..id)
	ReactionBar_CheckFaction()
end

function ReactionBar_SetRangeBar(id)
	id = tonumber(id)
	if (id == nil) then
		DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_NIL)
		ReactionBar_Status()
		return
	end
	
	if (id >= REACTIONBAR_MIN_BAR_ID and id <= REACTIONBAR_MAX_BAR_ID) then
		ReactionBarOptions[realmName][playerName].rangeBar = id
	end
	DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_RANGEBAR.." "..id)
	ReactionBar_CheckFaction()
end

function ReactionBar_SetNoTargetBar(id)
	id = tonumber(id)
	if (id == nil) then
		DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_NIL)
		ReactionBar_Status()
		return
	end
	
	if (id >= REACTIONBAR_MIN_BAR_ID and id <= REACTIONBAR_MAX_BAR_ID) then
		ReactionBarOptions[realmName][playerName].noTargetBar = id
	end
	DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_NOTARGETBAR.." "..id)
	ReactionBar_CheckFaction()
end

function ReactionBar_SetExtraBar(id)
	id = tonumber(id)
	if (id == nil) then
		DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_NIL)
		ReactionBar_Status()
		return
	end
	
	if (id >= REACTIONBAR_MIN_BAR_ID and id <= REACTIONBAR_MAX_BAR_ID) then
		ReactionBarOptions[realmName][playerName].extraBar = id
	end
	DEFAULT_CHAT_FRAME:AddMessage(REACTIONBAR_NAME_PRINT.." "..REACTIONBAR_EXTRABAR.." "..id)
	ReactionBar_CheckFaction()
end

function ReactionBar_SetActionPage(page)
	if (CURRENT_ACTIONBAR_PAGE ~= page) then
		CURRENT_ACTIONBAR_PAGE = page
		ChangeActionBarPage()
	end
end

function ReactionBar_OnUpdate(elapsed)
	if not initialised then
		return
	end

	updateInterval = updateInterval - elapsed
	if (updateInterval > 0) then
		return
	end
	updateInterval = 0.1

	ReactionBar_CheckFaction(1)
end

function ReactionBar_IsEnemy()
	if (duelStarted and duelTarget == UnitName("target")) then
		return true
	else
		if ( UnitPlayerControlled("target") ) then
			if (UnitCanAttack("target", "player") or UnitCanAttack("player", "target") ) then
				return true
			end
		elseif (not UnitIsDead("target") ) then
			if ( UnitIsTapped("target") and not UnitIsTappedByPlayer("target") ) then
				return true
			else
				local reaction = UnitReaction("target", "player")
				if (reaction and reaction <= 4 ) then
					return true
				end
			end
		end
	end
	return false
end